import { world } from '@minecraft/server';
import { ActionFormData, ModalFormData } from '@minecraft/server-ui';

class BypassSystem {
    constructor() {
        this.initDynamicProperties();
    }

    // Initialize dynamic properties for the kick message, toggle, and bypass lists
    initDynamicProperties() {
        if (!world.getDynamicProperty('KickMessage')) {
            world.setDynamicProperty('KickMessage', 'You have been kicked!');
        }
        if (!world.getDynamicProperty('BypassTags')) {
            world.setDynamicProperty('BypassTags', JSON.stringify([]));
        }
        if (!world.getDynamicProperty('BypassPlayers')) {
            world.setDynamicProperty('BypassPlayers', JSON.stringify([]));
        }
        if (world.getDynamicProperty('SystemToggle') === null) {
            world.setDynamicProperty('SystemToggle', true); // Default is enabled
        }
    }

    // Main menu with options
    async showMainMenu(player) {
        const toggleStatus = world.getDynamicProperty('SystemToggle') ? 'Enabled' : 'Disabled';
        const form = new ActionFormData()
            .title('Main Menu')
            .button('Edit Kick Message')
            .button('Bypass Tags/Players')
            .button(`Shutdown\n[${toggleStatus}]`);

        const response = await form.show(player);

        switch (response.selection) {
            case 0:
                this.editKickMessage(player);
                break;
            case 1:
                this.bypassMenu(player);
                break;
            case 2:
                this.toggleSystem(player);
                break;
        }
    }

    // Method to edit the kick message
    async editKickMessage(player) {
        const kickMessage = world.getDynamicProperty('KickMessage');
        const form = new ModalFormData()
            .title('Edit Kick Message')
            .textField('Kick Message', '', kickMessage);

        const data = await form.show(player);
        if (!data.canceled) {
            world.setDynamicProperty('KickMessage', data.formValues[0]);
            player.sendMessage('Kick message updated!');
        }
    }

    // Method for the bypass menu
    async bypassMenu(player) {
        const form = new ActionFormData()
            .title('Bypass Tags/Players')
            .button('List Bypass Tags/Players')
            .button('Add Tag/Player');

        const response = await form.show(player);

        if (response.selection === 0) {
            this.listBypassTagsAndPlayers(player);
        } else if (response.selection === 1) {
            this.addBypassTagOrPlayer(player);
        }
    }

    // Method to list bypass tags and players
    async listBypassTagsAndPlayers(player) {
        const tags = JSON.parse(world.getDynamicProperty('BypassTags'));
        const players = JSON.parse(world.getDynamicProperty('BypassPlayers'));

        const form = new ActionFormData()
            .title('Bypass List');

        // Add bypass tags to the menu
        for (const tag of tags) {
            form.button(`${tag}\n[Tag]`);
        }

        // Add bypass players to the menu
        for (const playerName of players) {
            form.button(`${playerName}\n[Player]`);
        }

        if (tags.length === 0 && players.length === 0) {
            form.button('No bypasses added.');
        }

        const response = await form.show(player);

        // If a bypass was selected, give options to edit or delete
        if (response.selection !== undefined) { // Check if a selection was made
            let selected;
            let isTag;

            // Determine if the selection is a tag or a player
            if (response.selection < tags.length) {
                selected = tags[response.selection];
                isTag = true; // Selected a tag
            } else {
                selected = players[response.selection - tags.length];
                isTag = false; // Selected a player
            }

            this.editOrDeleteBypass(player, selected, isTag);
        }
    }

    // Method to edit or delete a bypass
    async editOrDeleteBypass(player, bypassName, isTag) {
        const form = new ActionFormData()
            .title(`Manage Bypass: ${bypassName}`)
            .button('Edit')
            .button('Delete')
            .button('Cancel');

        const response = await form.show(player);

        if (response.selection === 0) {
            await this.editBypass(player, bypassName, isTag);
        } else if (response.selection === 1) {
            this.deleteBypass(player, bypassName, isTag);
        }
    }

    // Method to edit a bypass
    async editBypass(player, oldName, isTag) {
        const currentType = isTag ? 'Tag' : 'Player';
        const form = new ModalFormData()
            .title(`Edit Bypass: ${oldName}`)
            .textField('New Name:', '', oldName) // Text field for new name
            .dropdown('Change Type:', ['Tag', 'Player'], isTag ? 0 : 1); // Dropdown to select type

        const response = await form.show(player);
        if (!response.canceled) {
            const newName = response.formValues[0];
            const selectedType = response.formValues[1]; // 0 for Tag, 1 for Player

            // Remove the old bypass entry
            if (isTag) {
                this.removeFromBypassList(oldName, true);
            } else {
                this.removeFromBypassList(oldName, false);
            }

            // Add the new bypass entry with the selected type
            if (selectedType === 0) {
                this.addToBypassList(newName, true); // Add as tag
            } else {
                this.addToBypassList(newName, false); // Add as player
            }

            player.sendMessage(`Bypass updated from ${oldName} to ${newName} as a ${selectedType === 0 ? 'Tag' : 'Player'}.`);
        }
    }

    // Method to remove from the bypass list based on type
    removeFromBypassList(name, isTag) {
        const tags = JSON.parse(world.getDynamicProperty('BypassTags'));
        const players = JSON.parse(world.getDynamicProperty('BypassPlayers'));

        if (isTag) {
            const index = tags.indexOf(name);
            if (index !== -1) {
                tags.splice(index, 1); // Remove the tag
            }
        } else {
            const index = players.indexOf(name);
            if (index !== -1) {
                players.splice(index, 1); // Remove the player
            }
        }

        // Save updated lists back to dynamic properties
        world.setDynamicProperty('BypassTags', JSON.stringify(tags));
        world.setDynamicProperty('BypassPlayers', JSON.stringify(players));
    }

    // Method to delete a bypass
    deleteBypass(player, bypassName, isTag) {
        this.removeFromBypassList(bypassName, isTag);
        player.sendMessage(`${bypassName} has been deleted from the bypass list.`);
    }

    // Method to add a bypass tag or player
    async addBypassTagOrPlayer(player) {
        const form = new ActionFormData()
            .title('Add Bypass')
            .button('Add Tag')
            .button('Add Player');

        const response = await form.show(player);
        const isTag = response.selection === 0;

        const type = isTag ? 'tag' : 'player';
        const inputForm = new ModalFormData()
            .title(`Add ${type}`)
            .textField(`Enter ${type} name:`, '', '');

        const inputResponse = await inputForm.show(player);
        if (!inputResponse.canceled) {
            const name = inputResponse.formValues[0];
            this.addToBypassList(name, isTag);
            player.sendMessage(`${type.charAt(0).toUpperCase() + type.slice(1)} "${name}" has been added to the bypass list.`);
        }
    }

    // Method to add a tag or player to the bypass list
    addToBypassList(name, isTag) {
        const tags = JSON.parse(world.getDynamicProperty('BypassTags'));
        const players = JSON.parse(world.getDynamicProperty('BypassPlayers'));

        if (isTag) {
            tags.push(name); // Add tag
        } else {
            players.push(name); // Add player
        }

        // Save updated lists back to dynamic properties
        world.setDynamicProperty('BypassTags', JSON.stringify(tags));
        world.setDynamicProperty('BypassPlayers', JSON.stringify(players));
    }

    // Method to toggle the system on and off
    async toggleSystem(player) {
        const currentStatus = world.getDynamicProperty('SystemToggle');
        world.setDynamicProperty('SystemToggle', !currentStatus);
        player.sendMessage(`System has been ${currentStatus ? 'disabled' : 'enabled'}.`);
    }

    // Check if a player should be kicked upon joining
    handlePlayerJoin(player) {
        const kickMessage = world.getDynamicProperty('KickMessage');
        const kickToggle = world.getDynamicProperty('SystemToggle');
        const bypassPlayers = JSON.parse(world.getDynamicProperty('BypassPlayers'));

        if (kickToggle && !bypassPlayers.includes(player.name)) {
            player.runCommandAsync(`kick "${player.name}" ${kickMessage}`);
        }
    }
}

export const MP = new BypassSystem();

world.afterEvents.playerSpawn.subscribe((data) => {
    if (data.player && data.initialSpawn === true) {
        MP.handlePlayerJoin(data.player);
    }
});
