import { world, system } from '@minecraft/server';

const DefaultHomes = world.getDynamicProperty('HomesLimit')
const HomesEnabled = world.getDynamicProperty('HomesEnabled')
const MAX_HOMES = DefaultHomes;
const HOME_ENABLED = HomesEnabled; // true by default

class HomeSystem {
    addHome(player, homeName) {
        if (!HOME_ENABLED) {
            player.sendMessage("§cHome commands are currently disabled.");
            return;
        }

        const playerHomes = this.getHomes(player);
        if (playerHomes.length >= MAX_HOMES) {
            player.sendMessage(`§cYou can only have up to ${MAX_HOMES} homes.`);
            return;
        }

        const location = { x: player.location.x, y: player.location.y, z: player.location.z };
        playerHomes.push({ name: homeName, location });
        this.saveHomes(player, playerHomes);
        player.sendMessage(`§aHome "${homeName}" added.`);
    }

    deleteHome(player, homeName) {
        if (!HOME_ENABLED) {
            player.sendMessage("§cHome commands are currently disabled.");
            return;
        }

        const playerHomes = this.getHomes(player);
        const homeIndex = playerHomes.findIndex(home => home && home.name.toLowerCase() === homeName.toLowerCase());
        if (homeIndex === -1) {
            player.sendMessage(`§cNo home found with the name "${homeName}".`);
            return;
        }

        playerHomes.splice(homeIndex, 1);
        this.saveHomes(player, playerHomes);
        player.sendMessage(`§aHome "${homeName}" deleted.`);
    }

    listHomes(player) {
        if (!HOME_ENABLED) {
            player.sendMessage("§cHome commands are currently disabled.");
            return;
        }

        const playerHomes = this.getHomes(player);
        if (!playerHomes || playerHomes.length === 0) {
            player.sendMessage(`§cYou have no homes set.`);
            return;
        }

        player.sendMessage(`§aYour homes:`);
        playerHomes.forEach(home => {
            if (home && home.name) {
                player.sendMessage(`- ${home.name}`);
            } else {
                player.sendMessage(`§cFound an invalid home entry.`);
            }
        });
    }

    teleportToHome(player, homeName) {
        if (!HOME_ENABLED) {
            player.sendMessage("§cHome commands are currently disabled.");
            return;
        }

        const playerHomes = this.getHomes(player);
        const home = playerHomes.find(home => home && home.name.toLowerCase() === homeName.toLowerCase());
        if (!home) {
            player.sendMessage(`§cNo home found with the name "${homeName}".`);
            return;
        }

        // Check if location is defined before teleporting
        if (!home.location) {
            player.sendMessage(`§cInvalid home location for "${homeName}".`);
            return;
        }

        try {
            system.run(() => {
                      player.teleport(home.location);      
            })
            player.sendMessage(`§aTeleported to home "${homeName}".`);
        } catch (error) {
            player.sendMessage(`§cFailed to teleport to "${homeName}": ${error.message}`);
        }
    }

    getHomes(player) {
        const homesData = player.getDynamicProperty("homes");
        if (homesData) {
            try {
                const homes = JSON.parse(homesData);
                console.log(`Retrieved homes for ${player.name}:`, homes); // Log retrieved homes
                // Filter out any invalid entries before returning
                return homes.filter(home => home && home.name && home.location);
            } catch (e) {
                console.error(`Error parsing homes for ${player.name}:`, e);
                return []; // Return empty array if parsing fails
            }
        }
        return []; // Return empty array if homesData is null
    }

    saveHomes(player, homes) {
        console.log(`Saving homes for ${player.name}:`, homes); // Log homes being saved
        player.setDynamicProperty("homes", JSON.stringify(homes));
    }
}

const HomeManager = new HomeSystem();

world.beforeEvents.chatSend.subscribe(event => {
    const message = event.message.toLowerCase();
    const sender = event.sender;

    if (!HOME_ENABLED) return; // Ignore commands if homes are disabled

    if (message.startsWith("!addhome ")) {
        const homeName = message.substring(9).trim();
        HomeManager.addHome(sender, homeName);
        event.cancel = true;
    } else if (message.startsWith("!home ")) {
        const homeName = message.substring(6).trim();
        HomeManager.teleportToHome(sender, homeName);
        event.cancel = true;
    } else if (message === "!listhomes") {
        HomeManager.listHomes(sender);
        event.cancel = true;
    } else if (message.startsWith("!deletehome ")) {
        const homeName = message.substring(12).trim();
        HomeManager.deleteHome(sender, homeName);
        event.cancel = true;
    }
});
